/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://public.eu-egee.org/partners/ for details on 
 * the copyright holders.
 * For license conditions see the license file or
 * http://eu-egee.org/license.html
 *
 * Authors: 
 *      Paolo Badino <Paolo.Badino@cern.ch>>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Jozsef Kovacs <smith@lutra.sztaki.hu>
 *      Daniel Rodrigues <Daniel.Rodrigues@cern.ch>
 */
package org.glite.data.catalog.service.file;

import org.glite.data.catalog.service.AuthorizationException;
import org.glite.data.catalog.service.ExistsException;
import org.glite.data.catalog.service.FCEntry;
import org.glite.data.catalog.service.InternalException;
import org.glite.data.catalog.service.InvalidArgumentException;
import org.glite.data.catalog.service.NotExistsException;
import org.glite.data.catalog.service.PermissionEntry;
import org.glite.data.catalog.service.ServiceBase;
import org.glite.data.catalog.service.StringPair;
import org.glite.data.catalog.service.fas.FASBase;

/**
 * The middleware level API for the File Catalog component.  This is
 * the interface that this service exposes to the other middleware
 * components, which is not necessarily the final API seen by the
 * user.
 * <br>
 * <h3>Symlink Semantics</h3>
 *  <ul>
 *   <li> The only operation performed on a symlink itself is <i>unlink</i>.
 *   <li> All other operations are operations on the target LFN (change permission, read, stat, etc.).
 *   <li> The owner and group is set at creation time, it cannot be changed later.
 *   <li> Only the owner and group can unlink the symlink.
 *  </ul>
 *  <br>
 * @stereotype simple-interface
 */
public interface FileCatalog extends FASBase {
    
    //  commands updating the catalog
    
    /**
     * Make a set of new directories.
     * If the parent directories are not set yet, the createParents flag
     * will decide whether the parents should be created automatically.
     *
     * @param directories     The entries in the directory. This may be null for normal directories
     *                      but is mandatory for virtual directories.
     *                      The entry names may also be 'short', i.e. not start with a '/'
     *                      in which case they will be treated as names in the local directory.
     * @param createParents If set to true, instead of giving an error
     *                      if the parent directories do not exist, they will be created if
     *                      possible
     * @throws AuthorizationException No access right to create a directory or one of the parents.
     * @throws ExistsException		  The path or directory already exists. 
     * @throws NotExistsException	  One of the parent directories does not exists and
     * 								 'createParents' is false.
     * @throws InvalidArgumentException The path format is invalid
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of the path;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if directory already exists;
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check if Parents exists 
	 * <br> if it does not exit, behaviour depends on createParents argument :
	 * <br> createParents==0 > throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <br> createParents==1 > create parent directories recursively;
	 * <li> Check WRITE permissions on parent directory;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Insert new LFNEntry in the catalog
	 * <li> Insert new GUID to associate with the LFN
	 * <br> Permissions to the Guid are set by Principal Default, or Global Default 
	 * </ul>
	 *  
     */
    public void mkdir(String[] directories, boolean createParents, boolean copyPermFromParent) throws InternalException, InvalidArgumentException, AuthorizationException, ExistsException, NotExistsException;

    /**
     * Remove a directory.
     * 
     * @param directories The directory LFN to remove.
     * @throws AuthorizationException No access right to remove a directory or an entry in it.
     * @throws ExistsException		  The directory, to be removed, is not empty and 'recursive' flag is false.
     * @throws NotExistsException	  The directory, to be removed, does not exits.
     * @throws InvalidArgumentException The LFN format is not valid.
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
     * <li> Check the format of the directory LFN;
     * <br> if it is not correct, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if directory exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check REMOVE permission on the directory (in POSIX filesystems it is usually WRITE permission on the parent directory);
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Check if directory is empty;
	 * <br> if it is empty, removes the directory from the catalog.
	 * <br> if it is not empty, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * </ul>
     */
    public void rmdir(String[] directories) throws InternalException, AuthorizationException, ExistsException, NotExistsException;


    /**
     * Registers a new File Catalog entry with all status information specified.
     * 
     * @param entries FCEntry to register
     * @throws AuthorizationException    No access right to create new entry in the parent directory.
     * @throws ExistsException			 The file or guid already exists.
     * @throws NotExistsException		 One of the parent path elemenets does not exists.
     * @throws InvalidArgumentException		 The status information on the new entry is invalid.
     * @throws InternalException
     *
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check if FCEntry[] argument is valid;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check <i>type</i> in LFNStat to be file;
     * <br> if not, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN already exists
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check if GUID already exists (if given)
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check if parent directory exists
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check WRITE permissions on parent directory;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Insert the GUID information in the catalog
	 * <br> if it is not given, then create a GUID.
	 * <li> Insert new LFNEntry in the catalog.
	 * </ul>
	 *
     */
    public void createFile(FCEntry[] entries) throws InternalException, AuthorizationException, ExistsException, NotExistsException, InvalidArgumentException;

    /**
     * Create symlinks by passing source LFN (string1) and target LFN (string2) pairs.
     * Example: <code> ln -s b a </code> would be implemented as <code>symlink({"a", "b"}, true)</code>.
     * <br>
     * 
     * @param symlinks  Symlinks to register. 
     * @throws AuthorizationException    No access right to create new entry in the parent directory.
     * @throws ExistsException			 The file or Guid already exists.
     * @throws NotExistsException		 One of the parent path elemenets does not exists.
     * @throws InvalidArgumentException		 The status information on the new entry is invalid, e.g. type is not symlink.
     * @throws InternalException
     *      
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check if the format of the source and target LFNs are valid,
	 * <br> if not, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if source LFN (string1) already exists
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
     * <li> The target LFN (string2) does not have to exist at creation time (dangling symlink).
	 * <li> Check if parent directory exists
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check WRITE permissions on parent directory;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> See the applied permissions in the global description.
     * <li> Insert the symlink into the catalog.
	 * </ul>
     * 
	 */
    public void symlink(StringPair[] symlinks, boolean copyPermFromParent) throws InternalException, AuthorizationException, ExistsException, NotExistsException, InvalidArgumentException;

    /**
     * Delete an LFN, symlink or virtual directory. Use rmdir for normal directories.
     * 
     * @param lfns         The lfn to unlink
     * @throws AuthorizationException    No access right to delete the entry.
     * @throws NotExistsException		 One of lfns does not exists.
     * @throws InvalidArgumentException
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check if LFN string is valid;
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
 	 * <li> Check if LFN exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check if LFN is a directory
	 * <br> if it is a directory, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check REMOVE permissions on LFN;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Remove entry from associated tables in the catalog.
	 * </ul>
     */
    public void unlink(String[] lfns) throws InternalException, AuthorizationException, NotExistsException;

    /**
     * Renames a set of LFNs, symlinks or directories.
	 *
     * @param entries   {@link org.glite.data.catalog.service.StringPair StringPair} object, containing
     *                the old name as string1 and the new name as string2.
	 *
     * @throws AuthorizationException    No access right to create the new entry.
     * @throws ExistsException			 The destination entry already exists.
     * @throws InvalidArgumentException  Some part of the argument is invalid.
     * @throws NotExistsException		 The source entry does not exist.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of source and destination entry;
	 * <br> if any of them is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> If the destination is a link, then it has to be resolved and used as the target LFN (restart).
	 * <li> Check if source entry exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check REMOVE permission on the source;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Check WRITE permission on the (parent directory of) destination entry;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Check if the type of source entry is directory;
	 * <br> if it is a directory and destination is an existing directory, the directory is moved to the destination directory as a subdirectory.
	 * <br> if it is a directory and destination is an existing directory + a non-existing entryname and parent directories of the entries are different, the source directory is moved to the destination and source directory is renamed.
	 * <br> if it is a directory and destination is an existing file/link, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check if the type of source entry is file or link;
	 * <br> if it is a file/link and destination is an existing directory, the file/link is moved to the destination directory.
	 * <br> if it is a file/link and destination is an existing directory + a non-existing entryname and parent directories of the entries are different, the source file/link is moved to the destination directory and it is renamed.
	 * <br> if it is a file/link and destination is an existing file/link, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * </ul>
     */
    public void mv(StringPair[] entries) throws InternalException, AuthorizationException, ExistsException, NotExistsException;

    // status update methods

    /**
     * Update the modification time for the given entries.
     * 
     * @param lfns  The LFNs to change the time on
     * @param time  the new modification time to set
     * @return previous modification time
	 *
     * @throws AuthorizationException	No access right to update the status information.
     * @throws InvalidArgumentException	The specified time or the format of LFN is invalid.
     * @throws NotExistsException		The LFN does not exist.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of lfn;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if lfn exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check SET_METADATA permission on lfn;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Update modification time of the lfn in the catalog.
	 * </ul>
	 * </dt>
     */
    public void updateModifyTime(String[] lfns, long time) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Update the validity time for the given entries.
     * <br>
     * When current time ellapses creation time plus <i>validity time</i> the
     * file would be scheduled for removal by an additional service.
     * This means that setting <i>validity time</i> is implicitly
     * making decision on the removal of the file, therefor the
     * operation requires the <i>REMOVE</i> permission as well.
     * 
     * @param lfns  The LFNs to change the time on
     * @param time  the new validity time to set
     * @return previous validity time
	 *
     * @throws AuthorizationException	No access right to update the status information.
     * @throws InvalidArgumentException	The specified time or thw format of lfn is invalid.
     * @throws NotExistsException		The LFN does not exist.
	 * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of lfn;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if lfn exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check SET_METADATA and REMOVE permissions on lfn;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Update validity time of the lfn in the catalog.
	 * </ul>
	 * </dt>
     */
    public void updateValidityTime(String[] lfns, long time) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    // listing methods

    /**
     * Read the directory given in the path argument, restricted to
     * the pattern given in the second argument. The second argument
     * may be null, in which case all the directory is listed.  
     *<br>
     * The result contains a list of File Catalog entries, up to 'limit' of
     * them (given as the third argument). If limit is 0, an exception is
     * thrown. The offset is used to skip the first offset entries and start
     * listing with entry number offset+1. 
     * <br>
     * If <i>withPermissions</i> is not specified then FCEntry will not have
     * a Permission object (null) included, which speeds up listing considerably.
     * <br>
     * <i>pattern</i> allows glob like '?' and '*' characters, but no
     * '/', ranges, sets or other complex queries. Example: listing all files
     * starting with 'f', having at least 4 charactes is <code>f???*</code>.
     *
     * @param path    The path of the directory to query.
     * @param pattern The pattern to be mached. May be null.
     * @param limit   Up to limit result entries to be returned. May not be 0.
     * @param offset  Start with result entry at index offset+1. Offset starts with 0.
     * @param withPermissions fill permission field too
	 *
     * @return The returned structure contains an array of
     *         {@link org.glite.data.catalog.service.FCEntry FCEntry} objects.
     * @throws AuthorizationException	No access right to list the entries or status information.
     * @throws NotExistsException		The path does not exist.
     * @throws InvalidArgumentException
     * @throws InternalException			The offset or limit is larget than the absolute maximum allowed.
     *
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
     * <li> Check format of the pattern;
     * <br> if it is invalid, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check the limit;
     * <br> if it is not valid, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check format of directory;
	 * <br> if it is invalid, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if directory exists;
	 * <br> if it does not exist, throw {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check READ permission on the directory;
	 * <br> if permission is not granted, throw {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Iterate through the directory entries and match it against the pattern.
     * <li> Check LIST permission on each entry;
	 * <br> if permission is not granted, throw {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query directories and add them to the list to be returned. It is non-recursive.
	 * <li> Query files and add them to the list to be returned.
	 * <li> Query links and add them to the list to be returned.
     * <li> Return maximum <i>limit</i> number of entries starting from <i>offset</i>.
	 * </ul>
     *
     */
    public FCEntry[] readDir(String path, String pattern, long limit, long offset, boolean withPermissions) throws InternalException, AuthorizationException, NotExistsException,InvalidArgumentException;

    /**
     * Returns list of matching LFNs from all subdirectories of the
     * path given in the first argument. 
     * <br>
     * This method does not resolve * symlinks.
     * <br>
     * This method doesn not have <i>offset</i>, because traversal
     * of a directory tree is ambiquous, therefor the order of the
     * returned entries cannot defined.
     * 
     * @param startDir is for startDir
     * @param pattern is the element we are looking for.
     * @param limit  Up to limit result entries to be returned. May not be 0.
     * @return list of matching entries
     * 
     * @throws AuthorizationException	No access right to list the entries or status information.
     * @throws NotExistsException		The startDir does not exist.
     * @throws InvalidArgumentException
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
     * <li> Check format of the pattern;
     * <br> if it is invalid, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check the limit;
     * <br> if it is not valid, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if the path startDir is valid
	 * <li> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if directory exists
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check READ permission on <i>startDir</i> and on each sub-directory;
	 * <br> if permission is not granted, throw {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Get directories recursively under startDir and query their content.
  	 * <li> Query files and add them to the list to be returned.
     * <li> Return maximum <i>limit</i> number of entries.
	 * </ul>
	 *
     */
    public String[] locate(String startDir, String pattern, long limit) throws InternalException, AuthorizationException, NotExistsException;

    /**
     * Retrieves list of guids for given Lfns.
     * Both directories and files have guids.
     * 
     * @param lfns - lfns we want guid for
     * @return list of guids
     * 
     * @throws AuthorizationException	No access right to list the entries or status information.
     * @throws NotExistsException		The LFN does not exist.
     * @throws InvalidArgumentException 
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry::
	 * <ul>
	 * <li> Check if LFN is valid
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN exists
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on the LFN;
	 * <br> if permission is not granted, throw {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query catalog for GUID.
  	 * <li> Add response to the StringPair object (lfn,guid).
	 * </ul>
     * 
     * 
     */
    public StringPair[] getGuidForLfn(String[] lfns) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Retrieves list of lfns for given guid.
     * Does reverse lookup, returns full path.
     * 
     * @param guids list of guids we want lfns for
     * @return list of lfns/guids
     * 
     * @throws AuthorizationException	No access right to list the entries or status information.
     * @throws NotExistsException		The GUID does not exist.
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry::
	 * <ul>
	 * <li> Check if GUID is valid
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if GUID exists
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on the GUID;
	 * <br> if permission is not granted, throw {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query catalog for LFN.
  	 * <li> Add response to the StringPair object (guid,lfn).
	 * </ul>
     */
    public StringPair[] getLfnForGuid(String[] guids) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;


    /**
     * Get the detailed metadata about LFNs.
     * 
     * @param  lfns the array of strings to get the full status of
     * @return An array of {@link org.glite.data.catalog.service.FCEntry FCEntry} objects
     * 
     * @throws AuthorizationException	No access right to list the entries or status information.
     * @throws NotExistsException		The LFN does not exist.
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry::
	 * <ul>
	 * <li> Check if format of LFN is valid
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN exists
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on the LFN;
	 * <br> if permission is not granted, throw {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query catalog for Stat information.
  	 * <li> Query catalog for Permissions.
     * <li> Return the FCEntry objects.
	 * </ul>
     * 
     */
    public FCEntry[] getFileCatalogEntry(String[] lfns) throws InternalException, AuthorizationException, NotExistsException;

}
