/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 * 
 * Authors: 
 *      Paolo Badino <Paolo.Badino@cern.ch>>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Jozsef Kovacs <smith@lutra.sztaki.hu>
 *      Daniel Rodrigues <Daniel.Rodrigues@cern.ch> 
 */

package org.glite.data.catalog.service.meta;

import org.glite.data.catalog.service.*;

/**
 * The middleware level API for handling individual entries
 * in the Metadata Catalog component. This is the interface that the
 * service exposes to the other middleware components, which is not
 * necessarily the final API seen by the user.
 */
public interface MetadataCatalog extends MetadataBase, MetadataSchema {
    /**
     * Creates new items/entries in the catalog. 
     * 
     * <p>Each item/entry is associated with an existing schema, and though with the attributes which
     * belong to this schema. If an implementation does not care about having different schemas associated
     * with a group of entries in the catalog, then this attribute is ignored and the structure seen by
     * the client is a flat catalog.</p>
     * 
     * <p>The entries passed in the method are pairs of {item identifier, schema name}. The item identifier
     * is a string which must be unique in the catalog. The schema name is the name of the schema containing
     * the attributes to be associated with the new entry.</p>
     * 
     * @param entries The new entries to be inserted in the catalog as {item, schemaName} pairs.
     * 
     * @throws AuthorizationException No access for creating new entries/items.
     * @throws ExistsException The entry/item already exists in the catalog.
     * @throws NotExistsException The schema does not exist in the catalog.
     * @throws InvalidArgumentException The given string identifier for the new item/entry is invalid. 
     * @throws InternalException Any other error on the server side (i.e. database down).
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if client is authorized to create entries:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * </ul>
         * The following steps are performed for each entry passed in the array:
         * <ul>
         * <li>Check if the identifier given for the new entry is valid:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * <li>Check if the entry already exists in the catalog:
         * <br/>if it does, throw {@link org.glite.data.catalog.service.ExistsException}.</li>
         * <li>Check if the schema exists in the catalog - this step can be ignored if there is no schema
         * concept in the implementation:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Create the new entry in the catalog.</li>
         * </ul>
         * 
         * @see org.glite.data.catalog.service.meta
         * @see org.glite.data.catalog.service.meta.MetadataSchema
         * @see org.glite.data.catalog.service.StringPair
     */
    public void createEntry(StringPair[] entries) throws InvalidArgumentException, InternalException, ExistsException, NotExistsException, AuthorizationException;

    /**
     * Removes existing items/entries from the catalog.
     * 
     * <p>The concrete semantics of this operation are left to the implementation. It may be that all
     * entries should be deleted even if attributes are set, or that entries should only be deleted if no
     * associated attributes have values set.</p>
     * 
     * @param items The string identifiers of the items/entries to be removed from the catalog.
     * 
     * @throws AuthorizationException No access for deleting entries/items.
     * @throws NotExistsException The entry/item does not exist in the catalog.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if client is allowed to delete the items/entries from the catalog:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * </ul>
         * The following steps are performed for each item/entry:
         * <ul>
         * <li>Check if the item exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Remove the item from the catalog.</li>
         * </ul>
         * 
         * @see org.glite.data.catalog.service.meta
     */
    public void removeEntry(String[] items) throws AuthorizationException, NotExistsException, InternalException;
}
