/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 * 
 * Authors: 
 *      Paolo Badino <Paolo.Badino@cern.ch>>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Jozsef Kovacs <smith@lutra.sztaki.hu>
 *      Daniel Rodrigues <Daniel.Rodrigues@cern.ch>
 */

package org.glite.data.catalog.service.meta;

import org.glite.data.catalog.service.*;
import org.glite.data.catalog.service.fas.FASBase;

/**
 * The middleware level API for handling schemas in the Metadata Catalog component.
 * This is the interface that the service exposes to the other middleware components, which is not
 * necessarily the final API seen by the user.
 */
public interface MetadataSchema extends FASBase {
    /**
     * Creates a new schema in the catalog. 
     * 
     * <p>A schema is a group of attributes. A schema has a name so that this group can be easily refered
     * to. Each of this attributes will have its own name, type and value.</p>
     * 
     * <p>Schemas MUST be unique within the catalog.</p>
     * 
     * <p>An item/entry inside the catalog is associated with a schema. When a client queries or updates
     * the attributes of an item/entry, these must be contained in the associated schema.</p>
     * 
     * @param schemaName The name of the new schema.
     * @param attributes The list of attributes to associate with the new schema.
     * This is a list of {@link org.glite.data.catalog.service.Attribute} objects. Each object MUST
     * have the name and type fields filled in. The value field in the MAY be used for having default
     * values.
     * 
     * @throws AuthorizationException No access for client to create new schemas.
     * @throws ExistsException A schema with the same name already exists in the catalog, or there is
     * more than one attribute with the same name in the list of attributes given.
     * @throws InvalidArgumentException One of the attributes given is invalid. It may be due to an
     * invalid attribute name (i.e. empty), or the type given not being supported in the catalog.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if client is authorized to create new schemas:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * <li>Check if a schema with the same name already exists in the catalog:
         * <br/>if it does, throw {@link org.glite.data.catalog.service.ExistsException}.</li>
         * </ul>
         * The following additional steps should be performed for each attribute:
         * <ul>
         * <li>Check if the attribute type given is supported by the backend:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * <li>Check if the attribute name given already exists in the same schema in the request -
         * this is a check for repeated attributes:
         * <br/>if it does, throw {@link org.glite.data.catalog.service.ExistsException}.</li>
         * </ul>
         * Create the new schema in the catalog.
         * 
         * @see org.glite.data.catalog.service.Attribute
         * @see org.glite.data.catalog.service.meta
     */
    public void createSchema(String schemaName, Attribute[] attributes) throws AuthorizationException, ExistsException, InvalidArgumentException, InternalException;

    /**
     * Adds new attributes to an existing schema.
     * 
     * <p>Attributes MUST be unique within the whole catalog, so there is no way of creating an attribute
     * with a different type than an attribute with the same name in another schema.</p>
     * 
     * <p>The schemaName is the name of the schema where attributes should be added. The attributes 
     * parameter is simply a list of {@link org.glite.data.catalog.service.Attribute} objects where
     * the name and type field MUST be filled in, and the value field MAY be used by the implementation
     * to have default values for attributes in the schema.</p> 
     * 
     * @param schemaName The name of the schema to add the attributes.
     * @param attributes The attributes to be added to the schema. The name and type fields MUST
     * to be specified. The value field in the Attribute object MAY be used if a default value
     * is to exist.
     * 
     * @throws AuthorizationException No access for client to add attributes to the schema.
     * @throws NotExistsException The given schema does not exist in the catalog.
     * @throws ExistsException There is already one attribute in the schema with the same name as one of
     * the attributes in the list given. Or there is more than one attribute with the same name in the list.
     * @throws InvalidArgumentException One of the attributes given is invalid. It may be due to an
     * invalid attribute name (i.e. empty), or the type given is not supported in the catalog.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if client is authorized to add attributes to the schema:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * <li>Check if the given schema name exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * </ul>
         * The following additional steps are performed for each attribute:
         * <ul>
         * <li>Check if an attribute with the same name already exists in the schema or on of the other
         * attributes in the request:
         * <br/>if it does, throw {@link org.glite.data.catalog.service.ExistsException}.</li>
         * <li>Check if the name and type given for the attribute are valid in the catalog:
         * <br/>if one of them is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * </ul>
         * Add the new attributes to the schema.
         * 
         * @see org.glite.data.catalog.service.Attribute
         * @see org.glite.data.catalog.service.meta
     */
    public void addSchemaAttributes(String schemaName, Attribute[] attributes) throws AuthorizationException, NotExistsException, ExistsException, InvalidArgumentException, InternalException;

    /**
     * Removes attributes from an existing schema.
     * 
     * <p>The semantics of the removal process are up to the implementation to decide. It can remove the
     * attribute from the schema even if there are items/entries in the catalog with values set for it.
     * Or it can decide to remove only if there are no items/entries actually using this attribute at the
     * time - none has the value set. It can also decide to remove an attribute from a schema when no
     * item/entry in the catalog is associated with that same schema at the time of the request.</p>
     * 
     * <p>The schemaName is the name of the schema where the attributes will be removed. The attributeNames
     * is simply a list of the names of the attributes that should be removed from the schema.</p>
     * 
     * @param schemaName The name of the schema where the attributes should be removed.
     * @param attributeNames The names of the attributes to be removed from the schema.
     * 
     * @throws AuthorizationException No access for removing attributes from the schema.
     * @throws NotExistsException The given schema or one of the attributes given does not exist in the catalog.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if client is authorized to remove attributes from the schema:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * <li>Check if the given schema exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * </ul>
         * <br/>The following additional steps are performed for each attribute:
         * <ul>
         * <li>Check if the attribute exists in the given schema:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * </ul>
         * Remove the attributes from the schema.
         * 
         * @see org.glite.data.catalog.service.meta
     */
    public void removeSchemaAttributes(String schemaName, String[] attributeNames) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Drops an existing schema from the catalog.
     * 
     * <p>The semantics of this operation are up to the implementation. It may be that a schema can only
     * be droped if no items/entries in the catalog are associated with it at the time of the request. Or
     * it may be that schemas are dropped even when there are items/entries associated with them. This
     * would mean the existing metadata would be lost.</p>
     * 
     * <p>The schemaName is the name of the schema to be droped from the catalog.</p>
     * 
     * @param schemaName The name of the schema that should be droped from the catalog.
     * 
     * @throws AuthorizationException No access for client to drop the schema.
     * @throws NotExistsException The given schema does not exist in the catalog.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>The following steps are performed:
         * <ul>
         * <li>Check if the client is authorized to drop schemas:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * <li>Check if the given schema name exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Remove the schema from the catalog.</li>
         * </ul>
         * 
         * @see org.glite.data.catalog.service.meta
     */
    public void dropSchema(String schemaName) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Lists all existing schemas in the catalog.
     * 
     * <p>The list returned contains only the names of the schemas, not the whole description. To get the
     * details on each of the schemas in the catalog, an additional request should be made to
     * {@link #describeSchema}, where all the attributes and their descriptions will be returned.</p>
     * 
     * @return 	An array with the name of each existing schema in the catalog.
     * 			null if there are no schemas in the catalog.
     * 
     * @throws AuthorizationException No access to list schemas for the client.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>The following steps are performed:
         * <li>Check if client is authorized to list the existing schemas in the catalog:
         * <br/>if she is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * <li>Return the list of schemas.</li>
         * </ul>
         *
         * @see #describeSchema(String)
         * @see org.glite.data.catalog.service.meta
    */
    public String[] listSchemas() throws AuthorizationException, InternalException;

    /**
     * Get the full description of an existing schema in the catalog.
     * 
     * <p>A schema is described by the attributes it contains. Each of these attributes is represented as
     * a {@link org.glite.data.catalog.service.Attribute} object, so the description of a schema is simply
     * a list of {@link org.glite.data.catalog.service.Attribute} objects.</p>
     * 
     * <p>The schemaName is the name of the schema to return the description.</p>
     * 
     * @param schemaName The name of the schema to return the description.
     * @return A list of {@link org.glite.data.catalog.service.Attribute} objects fully describing all
     * elements of the schema. Each of these objects MUST have the name and type fields filled in. The
     * value type MAY also be used, but only when the implementation wants to get default values for
     * attributes in schemas. Returns null when the schema contains no attributes.
     * 
     * @throws AuthorizationException No access for client to get the schema description.
     * @throws NotExistsException The schema requested does not exist in the catalog.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * 
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>The following steps are performed:
         * <ul>
         * <li>Check if the client is authorized to get the schema description:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * <li>Check if the given schema exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Return the list of attributes for the schema.</li>
         * </ul>
         *
     */
    public Attribute[] describeSchema(String schemaName) throws AuthorizationException, NotExistsException, InternalException;
}
